
use strict;

package Bfe;

sub new {

  my ($class, $conf) = @_;

  my $area;

  if($conf->{xaos}) {		# parse xaos file
    open INPUT, $conf->{xaos} or die "must give file\n";
    
    my $type = 'mandelbrot';

    while(<INPUT>) {
      
      chop;
      
      unless(/^;/) {
	s/[)(]//g;
	my @parm = split / /;
	if($parm[0] eq 'juliaseed') {
	  $conf->{seed} = [@parm[1,2]];
	  $type = 'julia';
	} elsif($parm[0] eq 'maxiter') {
	  $conf->{maxiter} = $parm[1];
	} elsif($parm[0] eq 'view') {
	  $conf->{center} = [@parm[1,2], 0, 0];
	  print "center is ".join(' ',@{$conf->{center}})."\n";
	  $conf->{span} = [@parm[3,4]];
	} elsif($parm[0] eq 'formula' && $parm[1] =~ /mandel([0-9])?/) {
	  if(defined $1) {
	    $conf->{power} = $1;
	  } else {
	    $conf->{power} = 2;
	  }
	}
      }
    }

    my $ops;			# attractor options

    if($type eq 'mandelbrot') {
      $conf->{func} = "q^$conf->{power}+c";
    } else {
      $conf->{func} = "q^$conf->{power}+[".(join ' ',@{$conf->{seed}}).' 0 0]';
    }

    $conf->{vvect} = [1, 0, 0, 0] unless (defined $conf->{vvect});
    $conf->{hvect} = [0, 1, 0, 0] unless (defined $conf->{hvect});
    $conf->{bailout} = 500 unless (defined $conf->{bailout});    
  }
  
  $conf->{num_channels} = 3 unless (defined $conf->{num_channels});
  $conf->{num_bytes} = 1 unless (defined $conf->{num_bytes});  
  $conf->{num_threads} = 1 unless (defined $conf->{num_threads}); 
  $conf->{pallete} = 'pallete' unless (defined $conf->{pallete});
  $conf->{limit} = 30 unless (defined $conf->{limit}); 
  $conf->{maxiter} = 1000 unless (defined $conf->{maxiter}); 
  $conf->{bailout} = 500 unless (defined $conf->{bailout}); 
  unless (defined $conf->{size}) {
    if (defined $conf->{dpi}) {
      if(defined $conf->{page_size}) {
	$conf->{size}->[0] = $conf->{dpi}*$conf->{page_size}[0];
	$conf->{size}->[1] = $conf->{dpi}*$conf->{page_size}[1];
      } else {
	$conf->{size}->[0] = $conf->{dpi}*10.5;
	$conf->{size}->[1] = $conf->{dpi}*8;
	$conf->{page_size} = [10.5, 8];
      }
    } else {
      $conf->{size} = [100, 100];
    }
  }
  $conf->{outfile} = 'bfe.out' unless (defined $conf->{outfile});
  
  my $ops .= " -a" if(defined $conf->{attractor});
  $ops .= " -sd ",join(' ',@{$conf->{seed}}) if(defined $conf->{seed});

  bless $conf, $class;
}

sub generate {

  my $this = shift;

  warn "Bfe: num_channels not defined\n" 
    unless (defined $this->{num_channels});
  warn "Bfe: num_bytes not defined\n" unless (defined $this->{num_bytes});
  warn "Bfe: num_threads not defined\n" unless (defined $this->{num_threads});
  warn "Bfe: pallete not defined\n" unless (defined $this->{pallete});
  warn "Bfe: power not defined\n" unless (defined $this->{power});
  warn "Bfe: limit not defined\n" unless (defined $this->{limit});
  warn "Bfe: maxiter not defined\n" unless (defined $this->{maxiter});
  warn "Bfe: bailout not defined\n" unless (defined $this->{bailout});
  warn "Bfe: size not defined\n" unless (defined $this->{size});
  warn "Bfe: span not defined\n" unless (defined $this->{span});
  warn "Bfe: center not defined\n" unless (defined $this->{center});
  warn "Bfe: vvect not defined\n" unless (defined $this->{vvect});
  warn "Bfe: hvect not defined\n" unless (defined $this->{hvect});
  warn "Bfe: outfile not defined\n" unless (defined $this->{outfile});
  warn "Bfe: func not defined\n" unless (defined $this->{func});

  my $opt .= " -n $this->{area}" if(defined $this->{area});
  my $string = "~/src/bfe/bfe -ic $this->{num_channels} -ib $this->{num_bytes} -nt $this->{num_threads} -pf $this->{pallete} -p $this->{power} -l $this->{limit} -i $this->{maxiter} -b $this->{bailout} -size ".(join ' ',@{$this->{size}})." -span ".(join ' ', @{$this->{span}})." -center ".(join ' ',@{$this->{center}})." -hv ".(join ' ',@{$this->{vvect}})." -vv ".(join ' ', @{$this->{hvect}})." -f $this->{outfile} -F '$this->{func}' -IF c -s -c -a $opt";
  print $string;
  system $string;
}
1;
