use strict;

use Bfe::Graphics;

package Bfe::Html;

sub parse_html {
  
  my $conf_list = shift;
  
  for my $conf (@$conf_list) {
    
    my $in_string;
    $in_string = $conf->{text} if (defined $conf->{text});
    $in_string = `cat $conf->{infile}` if (defined $conf->{infile});
    die "need input" unless (defined $in_string);
    my @string = split /([<>])/, $in_string;
    
    my $in_tag = 0;
    
    my $font;
    my $text;
    my $size;
    my $pattern;
    my $shadow_height;
    my $bump_params;

    my $text_list = [];
    my $bar_list = [];
    
    my $width;
    my $height;
    my $offset;
    
    my $format = 'png';
    
    foreach (@string) {
      
      if($in_tag) {
	if(/^[ ]*render_text[ ]*\"([^\"]+)\"/) {
	  my $text = $1;
	  my $name = $1;
	  $name .= ".$font.$size.$pattern.";
	  $name .= (join '.', @$bump_params).'.' if (defined $bump_params);
	  $name .= "$format";
	  $name =~ s/ /_/g;
	  push @$text_list, {
			     name => $name,
			     font => $font,
			     text => $text,
			     size => $size,
			     pattern => $pattern,
			     shadow_height => $shadow_height,
			     bump_params => $bump_params
			    }; 
	  $_ = "img src = \"$name\" alt = \"$text\" border = 0";
	} elsif (/^[ ]*render_bar/) {
	  my $name = "bar.$width.$height.$offset.$pattern.";
	  $name .= (join '.', @$bump_params).'.' if (defined $bump_params);
	  $name .= "$format";
	  $name =~ s/ /_/g;
	  push @$bar_list, {
			    name => $name,
			    width => $width,
			    height => $height,
			    offset => $offset,
			    pattern => $pattern,
			    bump_params => $bump_params
			   };
	  $_ = "img src = \"$name\" border = 0";
	} elsif (/^[ ]*set_text_font[ ]*\"([^\"]+)\"/) {
	  $font = $1;
	  $_ = "";
	} elsif (/^([ ]*set_text_size)/) {
	  s/$1//;
	  s/ //g;
	  $size = $_;
	  $_ = "";
	} elsif (/^([ ]*set_text_shadow_height)/) {
	  s/$1//;
	  s/ //g;
	  $shadow_height = $_;
	  $_ = "";
	} elsif (/^([ ]*set_bar_width)/) {
	  s/$1//;
	  s/ //g;
	  $width = $_;
	  $_ = "";
	} elsif (/^([ ]*set_bar_height)/) {
	  s/$1//;
	  s/ //g;
	  $height = $_;
	  $_ = "";
	} elsif (/^([ ]*set_bar_offset)/) {
	  s/$1//;
	  s/ //g;
	  $offset = $_;
	  $_ = "";
	} elsif (/^[ ]*set_pattern[ ]*\"([^\"]+)\"/) {
	  $pattern = $1;
	  $_ = "";
	} elsif (/^([ ]*set_bump_params)/){
	  s/$1//;
	  eval "\$bump_params = $_";
	  die $@ if $@;
	  $_ = "";
	} elsif (/^[ ]*unset_bump_params/){
	  undef $bump_params;
	  $_ = "";
	} elsif (/^([ ]*set_format)/) {
	  s/$1//;
	  s/ //g;
	  $format = $_;
	  $format =~ s/[ ]//g;
	  $_ = "";
	}
      }
      
      $in_tag = 0 if(/>/);
      $in_tag = 1 if(/</);    
    }
    my $outstring = join '', @string;
    $outstring =~ s/<[ ]*>//g;
    
    Bfe::Graphics::make_bar($bar_list);
    Bfe::Graphics::make_text($text_list);
    
    open FH, ">$conf->{outfile}" or die "couldn't open $conf->{outfile}: $!";

    print FH $outstring;
    close FH;
  }
}
1;
