#!/usr/bin/perl
#
# ----------------------------------
#  fract2midi.pl
#
my $version = "0.10";
#
#  Jeremy Avnet <jeremy@theory.org>
#
#  17th February 1999
# ----------------------------------
#
#  usage: perl fract2midi.pl inputfile [outputfile]
#
#  inputfile - a whitespace separated list of x and y coordinates
#              the trajectory to make a midifile from
#
#  outputfile - midi will be written to this file
#               defaults to "fract.midi" if not specified
#

use MIDI::Simple;

use strict;

my $sc = new_score;
$sc->set_tempo(80000);
$sc->{duration}="sn";
$sc->patch_change(0,70);
$sc->patch_change(8,72);
$sc->{volume}="127";

open(INFILE, $ARGV[0]) or &usage;
undef $/;
my @coords = split /\s/, <INFILE>;
close INFILE;

my ($xhi, $xlo, $yhi, $ylo) = find_bounds(\@coords);

my ($x,$y);
while (@coords) {
  $x = shift @coords;
  $y = shift @coords;
  synch(\channel0($xhi, $xlo, $x),\channel8($yhi, $ylo, $y));
}

if ($ARGV[1]) {
  write_score($ARGV[1]);
} else {
  write_score("fract.midi");
}


sub channel0 {
  my ($xhi, $xlo, $x) = @_;
  $sc->n("c0",int((($x - $xlo)/($xhi - $xlo)) * 30.0) + 60);
}

sub channel8 {
  my ($yhi, $ylo, $y) = @_;
  $sc->n("c8",int((($y - $ylo)/($yhi - $ylo)) * 30.0) + 60);
}

sub find_bounds {
  my $z = shift;
  my ($reh, $rel, $iml, $imh, @z);
  
  print "\nFinding boundary of data set:\n";
  
  $reh = $rel = $$z[0];
  $imh = $iml = $$z[1];
  
  for (my $i = 2; $i <= $#$z; $i += 2) {
    ($reh = $$z[$i]) if ($$z[$i] > $reh);
    ($rel = $$z[$i]) if ($$z[$i] < $rel);
    ($imh = $$z[$i+1]) if ($$z[$i+1] > $imh);
    ($iml = $$z[$i+1]) if ($$z[$i+1] < $iml);
  }
  
  print "upper(re, im) = $reh, $imh\n";
  print "lower(re, im) = $rel, $iml\n\n";

  return($reh,$rel,$imh,$iml);
}

sub usage {
  print "\nFract2Midi v$version";
  print "\n----------------";
  print "\nusage: perl fract2midi.pl inputfile [outputfile]\n\n";
  exit(1);
}
