#!/usr/bin/perl

use strict;
use Gtk;
init Gtk;
my $version    = "v.alpha-1";
my $CONFIGFILE = ".gferc";

sub read_rc {
  my $bfeparms = shift;

  # set-up the initial variables
  #   if anything is not found in rc-file
  #   it will have been set here - eliminates
  #   need of figuring out what isn't in the rc-file.
  $$bfeparms{'threads'}    = "1";
  $$bfeparms{'binary'}     = "./pbfe";
  $$bfeparms{'palpath'}    = ".";
  $$bfeparms{'savpath'}    = "/tmp";
  $$bfeparms{'palette'}    = "palette";
  $$bfeparms{'limit'}      = "8";
  $$bfeparms{'channels'}   = "3";
  $$bfeparms{'size'}       = "250 250";
  $$bfeparms{'filename'}   = "bfeimg.png";
  $$bfeparms{'maxpow'}     = "2";
  $$bfeparms{'bailout'}    = "10";
  $$bfeparms{'iterations'} = "128";
  $$bfeparms{'var_x'}      = "re(z)";
  $$bfeparms{'var_y'}      = "im(z)";
  $$bfeparms{'var_u'}      = "|x|+i*|y|";
  $$bfeparms{'var_v'}      = "x+i*y";
  $$bfeparms{'function'}   = "g^2-g+c";
  $$bfeparms{'initval'}    = "0";
  $$bfeparms{'horz'}       = "1 0 0 0";
  $$bfeparms{'vert'}       = "0 1 0 0";
  $$bfeparms{'center'}     = "0 0 0 0";
  $$bfeparms{'span'}       = "3 3";
  $$bfeparms{'extras'}     = "-c -s";

  open(RCFILE, "$ENV{HOME}/$CONFIGFILE")
    or print("Config file not found. Using defaults.\n");
  my @line;
  while (<RCFILE>) {
    next if /^\#/;         # skip comments
    chomp;
    @line = split / = /;
    $$bfeparms{$line[0]} = $line[1];
  }
  close RCFILE;
}

sub labeled_entry {
  my ($box, $labeltext, $initval, $spacing) = @_;
  my $entrybox = new Gtk::HBox(0,0);
  my $label    = new Gtk::Label($labeltext);
  my $entry    = new Gtk::Entry;
  $entry->set_usize(40,22);
  $entry->set_text($initval);
  $box->pack_start($entrybox, 0, 1, $spacing);
  $entrybox->pack_start($label, 0, 0, 0);
  $entrybox->pack_start($entry, 1, 1, 0);
  return($entry);
}

sub notebook_create_pages {
  my($notebook,$bfeparms) = @_;
  my(
     $box,
     $entry,
     $pagebox,
     $pagelabel,
     $pagelabelbox
    );
  my $separator = new Gtk::HSeparator();

  my $popupmenubox   = new Gtk::HBox( 0, 0 );
  my $popupmenulabel = new Gtk::Label("Where are the popups?") ;
  $popupmenubox->pack_start( $popupmenulabel, 0, 1, 0 );
  $popupmenubox->show_all();


  # S y s t e m 
  $pagelabel    = new Gtk::Label "System";
  $pagelabelbox = new Gtk::HBox(0,0);
  $pagelabelbox->pack_start($pagelabel, 0, 1, 0);

  $pagebox      = new Gtk::VBox(0,0);
  $$bfeparms{'threads'} = labeled_entry($pagebox," threads:",$$bfeparms{'threads'},3);
  $$bfeparms{'binary'}  = labeled_entry($pagebox," binary:",$$bfeparms{'binary'},3);

  $box = new Gtk::HBox(1,1);
  $$bfeparms{'palpath'} = labeled_entry($box," pallete path:",$$bfeparms{'palpath'},3);
  $$bfeparms{'savpath'} = labeled_entry($box," save path:",$$bfeparms{'savpath'},3);
  $pagebox->pack_start($box, 0, 0, 0);  

  $$bfeparms{'extras'} = labeled_entry($pagebox," extra switches: ",$$bfeparms{'extras'},3);

  $notebook->append_page_menu($pagebox, $pagelabelbox, $popupmenubox);
  $pagelabelbox->show_all;
  $pagebox->show_all();


  # I m a g e
  $pagelabel    = new Gtk::Label "Image";
  $pagelabelbox = new Gtk::HBox(0,0);
  $pagelabelbox->pack_start($pagelabel, 0, 1, 0);

  $pagebox = new Gtk::VBox(0,0);

  $box = new Gtk::HBox(1,1);
  $$bfeparms{'palette'} = labeled_entry($box," palette:",$$bfeparms{'palette'},0);
  $pagebox->pack_start($box,0,0,0); 

  $box = new Gtk::HBox(1,1);
  $$bfeparms{'limit'}    = labeled_entry($box, " limit:",$$bfeparms{'limit'});
  $$bfeparms{'channels'} = labeled_entry($box," channels:",$$bfeparms{'channels'},0);
  $pagebox->pack_start($box,0,0,6); 

  $box = new Gtk::HBox(1,1);
  $$bfeparms{'size'}     = labeled_entry($box," size:",$$bfeparms{'size'},0);
  $$bfeparms{'filename'} = labeled_entry($box," filename:",$$bfeparms{'filename'},0);
  $pagebox->pack_start($box,0,0,0); 

  $notebook->append_page_menu($pagebox, $pagelabelbox, $popupmenubox);
  $pagelabelbox->show_all;
  $pagebox->show_all();


  # F u n c t i o n
  $pagelabel    = new Gtk::Label "Function";
  $pagelabelbox = new Gtk::HBox(0,0);
  $pagelabelbox->pack_start($pagelabel, 0, 1, 0);

  $pagebox = new Gtk::VBox(0,0);

  $box = new Gtk::HBox(1,1);
  $$bfeparms{'maxpow'}     = labeled_entry($box," maxpow:",$$bfeparms{'maxpow'},0);
  $$bfeparms{'bailout'}    = labeled_entry($box," bailout:",$$bfeparms{'bailout'},0);
  $$bfeparms{'iterations'} = labeled_entry($box," iterations:",$$bfeparms{'iterations'},0);
  $pagebox->pack_start($box, 0, 0, 0);
  $pagebox->pack_start($separator, 0, 0, 4);

  $box = new Gtk::HBox(1,1);
  $$bfeparms{'var_x'} = labeled_entry($box," Let x(q)=",$$bfeparms{'var_x'},0);
  $$bfeparms{'var_y'} = labeled_entry($box," Let y(q)=",$$bfeparms{'var_y'},0);
  $pagebox->pack_start($box, 0, 0, 0);  

  $box = new Gtk::HBox(1,1);
  $$bfeparms{'var_u'} = labeled_entry($box," Let u(q,x,y)=",$$bfeparms{'var_u'},0);
  $$bfeparms{'var_v'} = labeled_entry($box," Let v(q,x,y)=",$$bfeparms{'var_v'},0);
  $pagebox->pack_start($box, 0, 0, 6);  

  $box = new Gtk::HBox(1,1);
  $$bfeparms{'function'} = labeled_entry($box," f(q,x,y,u,v)=",$$bfeparms{'function'},0);
  $$bfeparms{'initval'}  = labeled_entry($box," q0=",$$bfeparms{'initval'},0);
  $pagebox->pack_start($box, 0, 0, 0);  

  $notebook->append_page_menu($pagebox, $pagelabelbox, $popupmenubox);
  $pagelabelbox->show_all();
  $pagebox->show_all();


  # V i e w
  $pagelabel    = new Gtk::Label "View";
  $pagelabelbox = new Gtk::HBox(0,0);
  $pagelabelbox->pack_start($pagelabel, 0, 1, 0);

  $pagebox = new Gtk::VBox(0,0);
  $$bfeparms{'horz'}   = labeled_entry($pagebox," horz:",$$bfeparms{'horz'},3);
  $$bfeparms{'vert'}   = labeled_entry($pagebox," vert:",$$bfeparms{'vert'},3);
  $$bfeparms{'center'} = labeled_entry($pagebox," center:",$$bfeparms{'center'},3);
  $$bfeparms{'span'}   = labeled_entry($pagebox," span:",$$bfeparms{'span'},3);

  $notebook->append_page_menu($pagebox, $pagelabelbox, $popupmenubox);
  $pagelabelbox->show_all;
  $pagebox->show_all();
}


sub create_notebook {
  my(
     $notebookbox,
     $buttonbox,
     $button,
     $separator,
     $notebook,
     $omenu,
     $menu,
     $submenu,
     $menuitem,
     $group,
     $notebook_window,
     %bfeparms
    );

  read_rc(\%bfeparms);
  
  # $notebook_window = new Gtk::Window('toplevel');
  $notebook_window = new Gtk::Window;
  $notebook_window->set_title("GTK/Perl Fractal Explorer  $version");
  $notebook_window->signal_connect("destroy",
				   sub {
				     print("\nConfig file not saved purposely!!\n",
					   "Use the 'Quit' button if you want me to save it next time.\n",
					   "Enjoy the SIGSEGV...\n");
				     &Gtk::main_quit;
				   }
				   );
  $notebook_window->signal_connect("delete_event",\&Gtk::false);
  $notebook_window->border_width(0);
  
  $notebookbox = new Gtk::VBox 0, 0;
  $notebook_window->add($notebookbox);
  
  $notebook = new Gtk::Notebook;
  $notebook->set_tab_pos("left");
  $notebookbox->pack_start($notebook, 1, 1, 0);
  $notebook->border_width(10);
  
  notebook_create_pages($notebook,\%bfeparms);
  
  $buttonbox = new Gtk::HBox( 0, 0 );
  $buttonbox->border_width(5);
  $notebookbox->pack_start($buttonbox, 0, 0, 0);
  
  $button = new Gtk::Button( 'Quit' );
  $button->signal_connect( 'clicked', 
			   sub { 
			     my $key;
			     open(RCFILE, ">$ENV{HOME}/$CONFIGFILE")
			       or die("Could not open rcfile: $ENV{HOME}/$CONFIGFILE\n");
			     foreach $key (sort keys %bfeparms) {
			       print RCFILE "$key = ", $bfeparms{$key}->get_text, "\n";
			     }
			     close RCFILE;
			     &Gtk::main_quit;
			   }
			 );
  $buttonbox->pack_end( $button, 1, 1, 5);
  
  $button = new Gtk::Button( 'Generate' );
  $button->signal_connect( 'clicked', 
			   sub {
			     my ($function, $var, $varfcn1, $varfcn2, $runline);
			     $function = $bfeparms{'function'}->get_text;
#			     foreach $var (keys %bfeparms) {
#			       if ($var =~ m/var_(\w)/) {
#				 $varfcn = $bfeparms{$1}->get_text;
#				 $function =~ s/$2/$varfcn/g;
#			       }
#			     }
			     foreach $var ('u','v') {
			       $varfcn1 = $bfeparms{"var_$var"}->get_text;
			       $varfcn2 = $bfeparms{'var_x'}->get_text;
			       $varfcn1 =~ s/x/($varfcn2)/g;
			       $varfcn2 = $bfeparms{'var_y'}->get_text;
			       $varfcn1 =~ s/y/($varfcn2)/g;
			       $function =~ s/$var/($varfcn1)/g;
			     }
			     foreach $var ('x','y') {
			       $varfcn1 = $bfeparms{"var_$var"}->get_text;
			       $function =~ s/$var/($varfcn1)/g;
			     }
			     $runline = 
			       sprintf("%s -nt %s -ic %s -pf %s/%s -l %s -i %s -p %s -b %s -IF %s -F '%s' -center %s -span %s -hv %s -vv %s -size %s %s -f %s/%s &", 
				       $bfeparms{'binary'}->get_text,
				       $bfeparms{'threads'}->get_text,
				       $bfeparms{'channels'}->get_text,
				       $bfeparms{'palpath'}->get_text,
				       $bfeparms{'palette'}->get_text,
				       $bfeparms{'limit'}->get_text,
				       $bfeparms{'iterations'}->get_text,
				       $bfeparms{'maxpow'}->get_text,
				       $bfeparms{'bailout'}->get_text,   
				       $bfeparms{'initval'}->get_text,
				       $function,
				       $bfeparms{'center'}->get_text,    
				       $bfeparms{'span'}->get_text,      
				       $bfeparms{'horz'}->get_text,     
				       $bfeparms{'vert'}->get_text,     
				       $bfeparms{'size'}->get_text,    
				       $bfeparms{'extras'}->get_text,
				       $bfeparms{'savpath'}->get_text,  
				       $bfeparms{'filename'}->get_text, 
				       $bfeparms{'savpath'}->get_text,  
				       $bfeparms{'filename'}->get_text); 
			     print "\n$runline\n";
			     system $runline;
			   }
			 );
  $buttonbox->pack_end( $button, 1, 1, 5);
  $notebook_window->show_all();
}

&create_notebook;
Gtk::main Gtk;
exit(0);
