#ifndef PBFE_H
#define PBFE_H

#include <libquat.h>

#ifdef USE_PNG
#include <png.h>
#endif

#include "pvwin.h"

#ifdef USE_THREADS
#include "prioq.h"
#endif

#define COLOR
#define PALLETE_SIZE 4

#define PIXEL unsigned char

#define WYDE_R(a) (*((WYDE*)(a)))
#define WYDE_G(a) (*(((WYDE*)(a))+1))
#define WYDE_B(a) (*(((WYDE*)(a))+2))

#define BYTE_R(a) (*((BYTE*)(a)))
#define BYTE_G(a) (*(((BYTE*)(a))+1))
#define BYTE_B(a) (*(((BYTE*)(a))+2))

#define WYDE unsigned short
#define BYTE unsigned char


typedef struct {
    int height;			// image height in pixels
    int width;			// image width in pixels
    double bailout;		// escape radius
    int maxiter;		// maxium iterations
    int maxpow;			// largest power in polynomial (for coloring)
    char filename[64];		// name of output file
    quat crosshair;		// center of image
    double hspan, vspan;	// distance covered in quaternion space
    quat min;			// minimum corner coords
    quat vstep, hstep;		// how much to step in each direction
    int limit;			// how often to cycle pallete
#ifdef USE_THREADS
    Queue<PIXEL> * q;		// priority queue to hold completed lines
    int currblock;		// which block needs to be done next?
    int num_threads;
    int currthread;		// for assiging thread numbers
    sem_t diverg_mutex;
#endif
    eq_table * table;		// for function iteration
    BYTE * pallete;		// array of pallete nodes
    int pallete_size;		// number of nodes in pallete
    char * pallete_file;
    int image_bytes;		// number of bytes per channel
    int image_bits;		// image_bytes * 8
    int image_channels;		// either 1 or 3
    bool smooth;
    bool color_by_angle;
    libquat_iter_func f;	// iteration function
    libquat_iter_func_1 f1;	// iteration function
    bool compile;
    class preview_window * pwin;
    PIXEL ** image;
#ifdef USE_PNG
    png_structp png_ptr;	// png stuff
    png_infop info_ptr;
    FILE * png_file;
#endif
} Parameters;



#endif
