#include "pvwin.h"
#include "splat.h"
#include <gtk/gtk.h>
#include <time.h>

#define NUM_INSTRUMENTS 5
#define HALFPI 1.5707963268
#define START 5
#define EVENT_METHOD(i, x) GTK_WIDGET_CLASS(GTK_OBJECT(i)->klass)->x

extern void init_png(Parameters * parms);

#ifdef USE_THREADS 
extern void * start_threads(void*);
#endif
preview_window::preview_window(Parameters * _parms, GtkPreviewType type) {

    parms = _parms;

    printf("parms = %p\n", parms);

    s = new Splat(0.34, 0.0002, 200);

    attractor_mode = false;
    attractor_computing = false;
    sound_playing = false;

    r.x = 0;
    r.y = 0;
    r.width = parms->width;
    r.height = parms->height;

    iter_buffer = new int*[parms->width];
    for(int i = 0 ; i < parms->width ; i++) {
	iter_buffer[i] = new int[parms->height];	
	for(int j = 0 ; j < parms->height ; j++)
	    iter_buffer[i][j] = 0;
    }
    
    points = 0;

    // hspan, vspan, crosshair
    
    win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    prev = gtk_preview_new(type);
    gtk_preview_size(GTK_PREVIEW(prev), parms->width, parms->height);
    
    GtkWidget *hruler = gtk_hruler_new();
    gtk_ruler_set_range(GTK_RULER(hruler), parms->crosshair.s-(parms->hspan/2.0),parms->crosshair.s+(parms->hspan/2.0),parms->crosshair.s,0.1);
    gtk_ruler_draw_ticks(GTK_RULER(hruler));
    gtk_ruler_draw_pos(GTK_RULER(hruler));
    gtk_ruler_set_metric(GTK_RULER(hruler), GTK_PIXELS);
    gtk_widget_show(hruler);
 
    gtk_signal_connect_object( GTK_OBJECT(prev), "motion_notify_event",
			       (GtkSignalFunc)EVENT_METHOD(hruler, motion_notify_event),
			       GTK_OBJECT(hruler) );   
    //    connect_to_signal(prev->motion_notify_event, hruler->motion_notify_event);

    GtkWidget *vruler = gtk_vruler_new();
    gtk_ruler_set_range(GTK_RULER(vruler), parms->crosshair.s-(parms->hspan/2.0),parms->crosshair.s+(parms->hspan/2.0),parms->crosshair.s,0.1);
    gtk_ruler_draw_ticks(GTK_RULER(vruler));
    gtk_ruler_draw_pos(GTK_RULER(vruler));
    gtk_ruler_set_metric(GTK_RULER(vruler), GTK_PIXELS);
    gtk_widget_show(vruler);

    
    GtkWidget *table = gtk_table_new(2,2,0);
    GtkWidget * b = gtk_button_new();
    gtk_table_attach_defaults(GTK_TABLE(table), b, 0, 1, 0, 1);
    gtk_table_attach_defaults(GTK_TABLE(table), hruler,1,2,0,1);
    gtk_table_attach_defaults(GTK_TABLE(table), vruler,0,1,1,2);
    gtk_table_attach_defaults(GTK_TABLE(table), prev,1,2,1,2);
    gtk_widget_show(b);
    gtk_widget_show(table);
    gtk_widget_show(prev);

    gtk_container_add(GTK_CONTAINER(win), table);

    gtk_signal_connect_object( GTK_OBJECT(prev), "motion_notify_event",
			       (GtkSignalFunc)EVENT_METHOD(vruler, motion_notify_event),
			       GTK_OBJECT(vruler) );   

				// begin sound window schtuff

    GtkWidget * swin = gtk_window_new(GTK_WINDOW_DIALOG);
    GtkWidget * hbox = gtk_hbox_new(1, 0);
    GtkWidget * hbox2 = gtk_hbox_new(1, 0);
    tempo_adj = gtk_adjustment_new(0.5, 0.1, 1, 0.01, 0.01, 0.01);
    GtkWidget * tempo = gtk_hscale_new(GTK_ADJUSTMENT(tempo_adj));
    GtkWidget * table2 = gtk_table_new(3, 1, 0);

    gtk_container_set_border_width (GTK_CONTAINER (swin), 5);
    gtk_window_set_title(GTK_WINDOW(swin), "sound");

    GtkWidget * button1;
    GtkWidget * button2;
// [-0.825000 -0.285000] (syncopate)
    instruments = new GtkWidget *[NUM_INSTRUMENTS];

    for(int i = 0 ; i < NUM_INSTRUMENTS ; i++) {
	char buf[10];
	sprintf(buf, "%d", i+1);
	if(i)
	    button1 = gtk_radio_button_new_with_label(
		gtk_radio_button_group(GTK_RADIO_BUTTON(button2)),
		buf);
	else
	    button1 = gtk_radio_button_new_with_label(NULL, buf);
	
	instruments[i] = button1;
	gtk_box_pack_start(GTK_BOX(hbox), button1, 1, 1, 3);
	gtk_widget_set_usize(button1, 30, 30);
	gtk_widget_show(button1);
	button2 = button1;
    }
    
    overlap = new GtkWidget *[NUM_INSTRUMENTS];	// hackish

    for(int i = 0 ; i < NUM_INSTRUMENTS ; i++) {
	char buf[10];
	sprintf(buf, "%d", i+1);
	if(i)
	    button1 = gtk_radio_button_new_with_label(
		gtk_radio_button_group(GTK_RADIO_BUTTON(button2)),
		buf);
	else
	    button1 = gtk_radio_button_new_with_label(NULL, buf);
	
	overlap[i] = button1;
	gtk_box_pack_start(GTK_BOX(hbox2), button1, 1, 1, 3);
	gtk_widget_set_usize(button1, 30, 30);
	gtk_widget_show(button1);
	button2 = button1;
    }
    
    GtkWidget * frame = gtk_frame_new("instrument");
    gtk_container_add(GTK_CONTAINER(frame), hbox);
    gtk_widget_show(frame);

    gtk_table_attach_defaults(GTK_TABLE(table2), frame, 0, 1, 0, 1);

    frame = gtk_frame_new("overlap");
    gtk_container_add(GTK_CONTAINER(frame), hbox2);
    gtk_widget_show(frame);
    gtk_table_attach_defaults(GTK_TABLE(table2), frame, 0, 1, 1, 2);

    frame = gtk_frame_new("duration");
    gtk_container_add(GTK_CONTAINER(frame), tempo);
    gtk_widget_show(frame);
    gtk_table_attach_defaults(GTK_TABLE(table2), frame, 0, 1, 2, 3);

    gtk_container_add(GTK_CONTAINER(swin), table2);
    
    gtk_widget_show(hbox2);
    gtk_widget_show(hbox);
    gtk_widget_show(tempo);
    gtk_widget_show(table2);
    //    gtk_widget_set_usize(tempo, 50, 100);
    //    gtk_widget_set_usize(swin, 100, 200);
    gtk_widget_show(swin);

				// end sound window schtuff 

    c1 = parms->hspan/2-parms->crosshair.s;
    c2 = parms->vspan/2-parms->crosshair.a;
    c3 = parms->width/parms->hspan;
    c4 = parms->height/parms->vspan;
    c5 = 1/c3;
    c6 = 1/c4;

    keep_going = false;
}

void preview_window::grab_image() { // copies image for alpha blending 

    image = new PIXEL*[parms->height];
    for(int i = 0 ; i < parms->height ; i++) {
	image[i] = new PIXEL[parms->width*parms->image_bytes*parms->image_channels];
	memcpy(image[i], parms->image[i], parms->width*parms->image_bytes*parms->image_channels);
    }
}

#define limit 80

void preview_window::iterate_point(double x, double y) {
    
    double * v = new double[4];
    unsigned char color[3];
    v[0] = x;
    v[1] = y;
    v[2] = v[3] = 0;

    double scaler;
    double scaleg;

    if( points < limit/4 ) {
	scaler = 1-((double)points)/limit*4;
	scaleg = 1;
    } else if(points < limit/2) {
	scaler = 1-((double)limit/2-points)/limit*4;
	scaleg = 1;
    } else if(points < 3*limit/4) {
	scaleg = 1-((double)points-limit/2)/limit*4;
	scaler = 1;
    } else  {
	scaleg = 1-((double)limit-points)/limit*4;
	scaler = 1;
    }
//    printf("points = %d scale = %.2f\n", points, scale);

//    color[0] = 0xFF*scale;
//    color[1] = 255*((double)0/100);
//    color[2] = 0xFF;

    color[0] = 0xFF; 
    color[1] = 0xFF;
    color[2] = 0xFF; 

#define ITERS 8000

    draw_point(v[0], v[1], color, ITERS+1, 1.0);

//    v = parms->table->iter_init(v);
    (*parms->f1)(v, 1);

//     color[0] = 0xFF;
//     color[1] = 0;
//     color[2] = 0;

       //     draw_point(v[0], v[1], color);

//     for(int i = 0 ; i < 40 ; i++)
//    	v = parms->table->iter_1();

    color[0] = (unsigned char)(0xFF*scaler);//  red varying with time
    color[1] = (unsigned char)(0xFF*scaleg);//green varying with time
    
    double p[2];
    double grab[2];
    bool check = true;
    int grab_value = 400;
    int check_again = 0;
    double bailout = 40;
    unsigned char saved_red, saved_green; 

    saved_red = color[0]; 
    saved_green = color[1]; 

    for(int i = 1 ; i < ITERS ; i++) {
	p[0] = v[0];
	p[1] = v[1];

	(*parms->f1)(v, 0);

	if(v[0]*v[0]+v[1]*v[1] > bailout)
	    break;

	if(i == grab_value) {	// grab value for period checking
	    grab[0] = v[0];
	    grab[1] = v[1];
	} else if (i == check_again) { // if period found and at 2p check again
	    if(fabs(grab[0]-v[0]) < c5 && fabs(grab[1]-v[1]) < c6)
		i = ITERS - 100;
	} else if(i > grab_value) { // check for period with p = i-grab_value
	    if(fabs(grab[0]-v[0]) < c5 && fabs(grab[1]-v[1]) < c6)
		check_again = grab_value+ 2*(i - grab_value);
	}
	
	//color[2] = 255*((double)i/ITERS);
	if (i < ITERS/2){
	  //comment out in favor of continuous increase of blue
	  //color[2] = 255*((double)i/(ITERS/2)); //blue ranges from 0 to 255
	}
	if (i >= ITERS/2){
	  //red decreases with iteration,
	  //varying between 100% and 50% of original
	  color[0] = (unsigned char)(saved_red*(1.5-(double)i/(ITERS)));
	  //green decreases with iteration, 
	  //varying between 100% and 50% of original
	  color[1] = (unsigned char)(saved_green*(1.5-(double)i/(ITERS)));
	  //comment out in favor of continuous increase of blue
	  //color[2] = 255; //blue at maximum
	}
	//does continuous increase of blue
	color[2] = (unsigned char)(255*((double)i/(ITERS)));
	//debugging shit
	//printf("%d: %d %d %d \n", i, color[0], color[1], color[2]);
	draw_point(v[0], v[1], color, i, 0.5+0.4*((double)i/ITERS));
    }
    gtk_widget_queue_draw(prev);
    points++;
    if(points > limit)
	points = 0;
}
				// attractor mode
void preview_window::iterate_point_much(double x, double y) {
    
    double * v = new double[4];
    unsigned char color[3];
    v[0] = x;
    v[1] = y;
    v[2] = v[3] = 0;

    attractor_mode = true;
    attractor_computing = true;
    
    attractor = new float*[parms->height];

    for(int i = 0 ; i < parms->height ; i++) {
	attractor[i] = new float[parms->width];
	for(int j = 0 ; j < parms->width ; j++)
	    attractor[i][j] = 0;
    }
	
    color[0] = 0xFF; 
    color[1] = 0xFF;
    color[2] = 0xFF; 

    draw_point(v[0], v[1], color, ITERS+1, 1.0);

    (*parms->f1)(v, 1);		// iterate initial

    int iterates = 0;
    clock_t time1;
    clock_t time2;
    
    while(keep_going) {
	time1 = clock();
	for(int i = 0 ; i < 20000 ; i++) {	
	    (*parms->f1)(v, 0);	// iterate point
	    draw_attractor_point(v[0], v[1]);
	}
	time2 = clock();
	while (gtk_events_pending())
	  gtk_main_iteration();
 	gtk_widget_queue_draw(prev);
	iterates += 20000;
	printf("%d points ", iterates);	
	printf("took %.2f seconds\n", ((double)(time2-time1))/CLOCKS_PER_SEC);
	
    }
    x_coord = x;
    y_coord = y;
    delete [] v;
    attractor_computing = false;
}

void preview_window::sonify_attractor(double x, double y) {

    sound_playing = true;
    fp = popen("csound -g -d -odevaudio -Lstdin test.orc test.sco", "w");
    FILE * fp2 = fopen("output", "w");
    double * v = new double[4];

				// ymin is fucked up 
    int xmax = 0, xmin = parms->height, ymax = 0, ymin;
    double total = 0;

    int instrument;
    int mult;

    for(int i = 0 ; i < NUM_INSTRUMENTS ; i++) {
	if(GTK_TOGGLE_BUTTON(instruments[i])->active)
	    instrument = i+1;
	if(GTK_TOGGLE_BUTTON(overlap[i])->active)
	    mult = i+1;
    }

    ymin =  parms->width*parms->image_bytes*parms->image_channels;

				// determine attractor extents
    for(int i = 0 ; i < parms->height ; i++) {
	for(int j = 0 ; 
	    j < parms->width ;
	    j++) {
	    if(attractor[i][j] != 0) {
		total += attractor[i][j];
		if(i < xmin)
		    xmin = i;
		else if(xmax < i)
		    xmax = i;

		if(j < ymin)
		    ymin = j;
		else if(ymax < j)
		    ymax = j;
	    }
	}
    }
    
    double max_val = 0;

				// normalize to [0,1]
    for(int i = 0 ; i < parms->height ; i++) {
	for(int j = 0 ; 
	    j < parms->width ;
	    j++) {
	    if(attractor[i][j] != 0)
		attractor[i][j] /= total;	
	    if(attractor[i][j] > max_val)
		max_val = attractor[i][j];
	}
    }
				// attractor is now a probability
				// density funcion 
    
    v[0] = x;			
    v[1] = y;
    v[2] = v[3] = 0;

    (*parms->f1)(v, 1);		// iterate initial

    double octave = (xmax - xmin)*parms->hspan/(3*parms->width); // three octaves
    double yspan = ymax - ymin;;

//    printf("octave is %.2f\n", octave);

    for(int i = 0 ; i < 1000 ; i++)
	(*parms->f1)(v, 0);	// discard transients
// [-0.720000 -0.240000]
// [-0.615000 -0.435000]

    keep_going = true;
    int i = 240;
    double t = 0;
// control [-0.550000 -0.565000]

    char buf[2048];

    while(i--) {
	(*parms->f1)(v, 0);	// iterate point	
	double pitch = fabs(parms->crosshair.s-parms->hspan/2+
			    xmin*parms->hspan/parms->width-v[0])/octave;

	int img_x = (int)(rint((v[0]+c1)*c3)-1);
	int img_y = (int)(rint((v[1]+c2)*c4)-1);

	if(img_x > 0 && img_x < parms->width &&
	   img_y > 0 && img_y < parms->height) {
	    sprintf(buf, "i %d\t%.2f\t%.2f\t%.2f\t%.0f.%.2d\t%f\n",
		    instrument, t,
		    mult*GTK_ADJUSTMENT(tempo_adj)->value/**0.92*/, 
		    75+10*log10(log10(log10(attractor[img_x][img_y]*99/max_val+1)*49.5+1)*49.5+1),
		    floor(pitch)+6, (int)rint((pitch-floor(pitch))*12), 
		    0.3+0.7*((double)(img_y-ymin))/yspan); 
	    fputs(buf, fp);
	    fputs(buf, fp2);	    
	}
	

	t += GTK_ADJUSTMENT(tempo_adj)->value;
    }
    fflush(fp);
    fclose(fp2);
    printf("sonification done\n");
}

void preview_window::iterate_midi(double x, double y) {
  double * v = new double[4];
  FILE *TRAJECTORY;
  v[0] = x;
  v[1] = y;
  v[2] = v[3] = 0;
  
  TRAJECTORY = fopen("traj.tmp","w");
  (*parms->f1)(v,1);
  for (int i = 0; i < 50000; i++)
    (*parms->f1)(v, 0); 	// iterate point
  for(int i = 0 ; i < 20 ; i++) {
    (*parms->f1)(v, 0); 	// iterate point
    fprintf(TRAJECTORY,"%f %f\n",v[0],v[1]);
    printf("%f %f\n",v[0],v[1]);
  }
  fclose(TRAJECTORY);
  system("perl fract2midi.pl traj.tmp pbfe.midi");
}

void preview_window::draw_attractor_point(double x, double y) {
    
    double img_x, img_y;

				// back project point to image coords
    img_x = (x+c1)*c3;
    img_y = (y+c2)*c4;

	// splat over neibhors
	
    int start_x = (int)rint(img_x)-1;
    int start_y = (int)rint(img_y)-1;

//    printf("img (%.2f %.2f) start (%d %d)\n", img_x, img_y, start_x, start_y);
    int i;
    for(int j = start_y ; j < start_y + 3 ; j++) {
	if(j >= 0 && j < parms->height) {
	    for(i = start_x ; i < start_x + 3 ; i++)
		if(i >= 0 && i < parms->width) {
		    double f = s->area_under(img_x-i, img_y-j);
//		printf("%.5f ", f);
		    attractor[i][j] += f;//s->area_under(img_x-i, img_y-j);
		    PIXEL * pix = image[j]+i*3;
#if 0
		    get_attractor_color(attractor[j][i], pix);
#else
		    pix[0] = pix[1] = pix[2] = (unsigned char)(204*atan(0.008*attractor[i][j])/HALFPI);
    
//		    get_attractor_grey(attractor[i][j], pix);
#endif
//		    prev->draw_row(image[i]+j*3, i, j, 1);
// 		    gtk_preview_draw_row(GTK_PREVIEW(prev), 
// 					 image[i]+j*3, i, j, 1);
		}
//	    gtk_preview_draw_row(GTK_PREVIEW(prev), image[j]+start_x*3, start_x, j, 1);
#if 1
	    if(start_x + 3 < parms->width) {
		if(start_x >= 0)
		    gtk_preview_draw_row(GTK_PREVIEW(prev),
					 image[j]+start_x*3, start_x, j, 3);
		else if(start_x + 2 >= 0) {
		    gtk_preview_draw_row(GTK_PREVIEW(prev), 
					 image[j]+(start_x+1)*3, 
					 start_x+1, j, 2);
		    printf(".");
		} else if(start_x + 1 >= 0) {
		    gtk_preview_draw_row(GTK_PREVIEW(prev), 
					 image[j]+(start_x+2)*3, 
					 start_x+2, j, 1);
		    printf(".");
		}
	    } else if(start_x - 1 < parms->width) {
		    printf(".");
		gtk_preview_draw_row(GTK_PREVIEW(prev), 
				     image[j]+start_x*3, start_x, j, 2);
	    } else {
		    printf(".");
		gtk_preview_draw_row(GTK_PREVIEW(prev), 
				     image[j]+start_x*3, start_x, j, 1);
	    }
#endif	
	}
//	printf("\n");
    }
//    printf("\n");
}
    
void preview_window::get_attractor_grey(float weight, PIXEL * im) {

    im[0] = im[1] = im[2] = (unsigned char)(204*atan(0.008*weight)/HALFPI);

}
void preview_window::get_attractor_color(float weight, PIXEL * im) {
    
    double r, g, b;
    double theta1 = atan(0.002*weight);
    double theta2 = atan(0.01*weight)/HALFPI;

    hsv_to_rgb(theta1*220/HALFPI, 1.0/*0.5+theta2/2.0*/, theta2, &r, &g, &b);
    
    im[0] = (unsigned char)(255*r);
    im[1] = (unsigned char)(255*g);
    im[2] = (unsigned char)(255*b);
}
				// from gtk_colorsel
void preview_window::hsv_to_rgb(double h, double s, double v, 
				double *r, double* g, double* b) {
    int i;
    double f, w, q, t;

    if (s == 0.0)
    {
	if (h < 0.0 || h > 1.0)
        {
	    *r = v;
	    *g = v;
	    *b = v;
        }
    }
    else
    {
	if (h == 360.0)
	    h = 0.0;
	h = h / 60.0;
	i = (int)h;
	f = h - i;
	w = v * (1.0 - s);
	q = v * (1.0 - (s * f));
	t = v * (1.0 - (s * (1.0 - f)));
	switch (i)
        {
        case 0:
	    *r = v;
	    *g = t;
	    *b = w;
	    break;
        case 1:
	    *r = q;
	    *g = v;
	    *b = w;
	    break;
        case 2:
	    *r = w;
	    *g = v;
	    *b = t;
	    break;
        case 3:
	    *r = w;
	    *g = q;
	    *b = v;
	    break;
        case 4:
	    *r = t;
	    *g = w;
	    *b = v;
	    break;
        case 5:
	    *r = v;
	    *g = w;
	    *b = q;
	    break;
        }
    }
}
#if 0
PIXEL preview_window::get_attractor_red(float weight) {
    
#if 0
    return (unsigned char)(weight == 0 ? 0 : START+25*log(1+weight));
#else
    return (unsigned char)(weight == 0 ? 0 : START+(255-START)/HALFPI*atan(0.002*weight));
#endif
}

PIXEL preview_window::get_attractor_blue(float weight) {
    
//    weight -= 10000;

#if 0
    return (unsigned char)(weight == 0 ? 0 : START+25*log(1+weight));
#else
    if(weight > 10000)
	return((unsigned char)(128+5.04*pow(weight-10000,0.333333333333)));
    else {
	double fuck = pow(10000-weight,0.333333333333);
//	printf("fuck is %f\n", fuck);
	return((unsigned char)(128-5.84*fuck));
    }
#endif
}
#endif

void preview_window::draw_point(double x, double y, unsigned char * color,
				int iters, double alpha) {
    
    double img_x, img_y;
    int c_x, c_y;

				// back project point to image coords
    img_x = (x+c1)*c3;
    img_y = (y+c2)*c4;

    c_x = (int)rint(img_x);
    c_y = (int)rint(img_y);

    if(c_x >= 0 && c_y >= 0 && 
       c_x < parms->width && c_y < parms->height &&
       iter_buffer[c_x][c_y] <= iters) {
	iter_buffer[c_x][c_y] = iters;

	// splat over neibhors
	
	int n[4][2];
	n[0][0] = (int)floor(img_x);
	n[0][1] = (int)floor(img_y);

	n[1][0] = n[0][0]+1;
	n[1][1] = n[0][1];

	n[2][0] = n[0][0];
	n[2][1] = n[0][1]+1;

	n[3][0] = n[1][0];
	n[3][1] = n[2][1];

	bool inside[4];
	for(int i = 0 ; i < 4 ; i++)
	    if(n[i][0] < 0 || n[i][0] >= parms->width || 
	       n[i][1] < 0 || n[i][1] >= parms->height)
		inside[i] = false;
	    else 
		inside[i] = true;

	double dist[4];
#if 1
	double f1 = img_x - n[0][0];
	double f2 = img_x - n[1][0];
	double f3 = img_y - n[0][1];
	double f4 = img_y - n[2][1];

	double x_f = f1*f1;
	double x_c = f2*f2;
	double y_f = f3*f3;
	double y_c = f4*f4;

	dist[0] = 2-(x_f+y_f);
	dist[1] = 2-(x_c+y_f);
	dist[2] = 2-(x_f+y_c);
	dist[3] = 2-(x_c+y_c);
#else
	dist[0] = dist[1] = dist[2] = dist[3] = .25;
#endif

	double alphas[4];
	double sum = alpha/(dist[0]+dist[1]+dist[2]+dist[3]);
	for(int i = 0 ; i < 4 ; i++)
	    if(inside[i]) {
		alphas[i] = dist[i] * sum;	
#if 1
		PIXEL * fuck = image[n[i][1]]+n[i][0]*3;
		double fuck2 = 1-alphas[i];
		if(parms->image_bytes == 1) {
		    *fuck = (unsigned char)(fuck2**fuck+alphas[i]*color[0]);
		    if(parms->image_channels > 1) {
			fuck++;
			*fuck = (unsigned char)(fuck2**fuck+alphas[i]*color[1]);
			fuck++;
			*fuck = (unsigned char)(fuck2**fuck+alphas[i]*color[2]);
		    }
		} else {
// 		WYDE_R(image[img_y]+img_x) = ((1-alpha)*WYDE_R(image[img_y]+img_x)+
// 					      alpha*color[0]);
// 		if(parms->image_channels > 1) {
// 		    WYDE_G(image[img_y]+img_x) =((1-alpha)*WYDE_G(image[img_y]+img_x)+
// 						 alpha*color[1]);
// 		    WYDE_B(image[img_y]+img_x) =((1-alpha)*WYDE_B(image[img_y]+img_x)+
// 						 alpha*color[2]);
		}
#endif

	    }
	    
	if(inside[0] || inside[1])
	    gtk_preview_draw_row(GTK_PREVIEW(prev), image[n[0][1]]+n[0][0]*3+!inside[0],
			   n[0][0]+!inside[0], n[0][1], inside[0]+inside[1]);
	if(inside[2] || inside[3])
	    gtk_preview_draw_row(GTK_PREVIEW(prev), image[n[2][1]]+n[2][0]*3+!inside[2],
			   n[2][0]+!inside[2], n[2][1], inside[2]+inside[3]);
    }
}

gint preview_window::handle_move(GdkEventMotion * e) {

    double comp_x, comp_y;

//    vruler->motion_notify_event(e);

    if(e->state & GDK_BUTTON1_MASK && 
       !(e->state & GDK_SHIFT_MASK || keep_going)) {
	comp_x = ((double)e->x)/parms->width*parms->hspan+parms->crosshair.s-parms->hspan/2;
	comp_y = ((double)e->y)/parms->height*parms->vspan+parms->crosshair.a-parms->vspan/2;

	iterate_point(comp_x, comp_y);
    }
}

gint preview_window::handle_mouse_up(GdkEventButton * e) {


}

gint preview_window::handle_mouse_down(GdkEventButton * e) {

    double comp_x, comp_y;

    comp_x = e->x/parms->width*parms->hspan+parms->crosshair.s-parms->hspan/2;
    comp_y = e->y/parms->height*parms->vspan+parms->crosshair.a-parms->vspan/2;

    if(attractor_mode)
      return 0;

    if(e->state & GDK_SHIFT_MASK) {
      keep_going = true;
      iterate_point_much(comp_x, comp_y);
    } else if(e->state & GDK_CONTROL_MASK) {
      printf("control [%lf %lf]\n", comp_x, comp_y);
    } else if(e->state & GDK_BUTTON2_MASK) {
      printf("button2 [%lf %lf]\n", comp_x, comp_y);
    } else if(e->state & GDK_BUTTON3_MASK) {
      printf("button3 [%lf %lf]\n", comp_x, comp_y);
    } else if(e->state & GDK_BUTTON4_MASK) {
      printf("button4 [%lf %lf]\n", comp_x, comp_y);
    } else if(e->state & GDK_BUTTON5_MASK) {
      printf("button5 [%lf %lf]\n", comp_x, comp_y);
    } else if(e->state & GDK_LOCK_MASK) {           // caps lock + mouse click
      printf("lock [%lf %lf]\n", comp_x, comp_y);   
    } else if(e->state & GDK_MOD1_MASK) {
      printf("mod1 [%lf %lf]\n", comp_x, comp_y);
    } else if(e->state & GDK_MOD2_MASK) {           // num lock + mouse click
      iterate_midi(comp_x,comp_y);
    } else if(e->state & GDK_MOD3_MASK) {
      printf("mod3 [%lf %lf]\n", comp_x, comp_y);
    } else if(e->state & GDK_MOD4_MASK) {
      printf("mod4 [%lf %lf]\n", comp_x, comp_y);
    } else if(e->state & GDK_MOD5_MASK) {           // scroll lock + mouse click
#if 0

	gtk_widget_set_events(parms->pwin->prev, 
			    GDK_BUTTON_PRESS_MASK | 
			    GDK_KEY_PRESS_MASK |
			    GDK_POINTER_MOTION_MASK |
			    GDK_BUTTON_RELEASE_MASK);

      parms->table = new eq_table("q^2+c", "0",
				  parms->maxiter,
				  parms->bailout);
      if(parms->compile)
	parms->f = parms->table->compile();
      else
	parms->f = &eq_table_iter;
      parms->f1 = parms->table->compile_1();
      parms->pwin = new preview_window(parms, parms->image_channels == 3 ? 
				       GTK_PREVIEW_COLOR : GTK_PREVIEW_GRAYSCALE);
      connect_to_method(parms->pwin->prev->button_press_event, parms->pwin,
			&preview_window::handle_mouse_down);  
      connect_to_method(parms->pwin->prev->button_release_event, parms->pwin, 
			&preview_window::handle_mouse_up); 
      connect_to_method(parms->pwin->prev->motion_notify_event, parms->pwin,
			&preview_window::handle_move);
      connect_to_method(parms->pwin->key_press_event, parms->pwin, 
			&preview_window::handle_key);
      int nshades = parms->image_bytes == 1 ? 0xFF : 0xFFFF;
      parms->pwin->prev->set_color_cube(nshades, nshades, nshades, nshades);
      parms->pwin->show();
      parms->currthread = 0;
      parms->q = new Queue<PIXEL>(100);
      sem_init(&parms->diverg_mutex, 0, 1);
      pthread_t writer;
      pthread_attr_t attrs;		/* create threads */
      pthread_attr_init(&attrs);
      pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
      pthread_create(&writer, &attrs, start_threads, (void *)parms);
      parms->image = new PIXEL*[parms->height];
      //      gtk_timeout_add(1, display_fractal, (gpointer)parms);
      display_fractal(parms);
      //      gui.run();
      printf("\nCrosshair: (%f,%f)   Resolution: %f x %f\n", 
	     parms->crosshair.s, parms->crosshair.a, parms->hspan, parms->vspan);
      printf("mod5 [%lf %lf]\n", comp_x, comp_y);
#endif
    }
    else
      iterate_point(comp_x, comp_y);
}

gint preview_window::handle_key(GdkEventKey * e) {
  
    if(e->keyval == 32) {
	if(attractor_mode) 
	    keep_going = false;
	if(sound_playing) {
	    system("killall csound"); // -|-HACK-|-HACK-|-HACK-|-
	    pclose(fp);
	    sound_playing = false;
	    keep_going = false;
	}
    } else if(e->keyval == 'q') {
	if(sound_playing) {
	    system("killall csound"); // -|-HACK-|-HACK-|-HACK-|-
	    pclose(fp);
	}
	gtk_main_quit();
    } else if(e->keyval == 'c') {
       	if(attractor_mode && !attractor_computing) {
	    attractor_mode = false;

	    for(int i = 0 ; i < parms->height ; i++) 
		delete [] attractor[i];
	    delete [] attractor;
	}

	for(int i = 0 ; i < parms->height ; i++)
	    gtk_preview_draw_row(GTK_PREVIEW(prev),
				 parms->image[i], 0, i, parms->width);
      
	gtk_widget_draw(prev, &r);
	for(int i = 0 ; i < parms->height ; i++) {// clear buffer
	    for(int j = 0 ; j < parms->width ; j++)
		iter_buffer[i][j] = 0;
	    memcpy(image[i], parms->image[i], parms->height*parms->image_bytes*parms->image_channels);
	}
    } else if(e->keyval == 's') {
	if(attractor_mode && !sound_playing && !attractor_computing)
	    sonify_attractor(x_coord, y_coord);
    } else if(e->keyval == 'i') {
	init_png(parms);
	png_write_rows(parms->png_ptr, image, parms->height);
	png_write_end(parms->png_ptr, NULL);
	fclose(parms->png_file);
    }
}

