#!/usr/bin/perl

use Gimp;
use Bfe;
use Bfe::Graphics;
use strict;

sub make_span {
  
  my $maxiter = 1500;
  my @seed;
  my @view;
  my $pages_vert = 1;
  my $dpi = 60;
  my $out = 1;			# set to 1 to get bzip2'd printer food
  my $x_do_only = 1;
  my $y_do_only = 2;
  my $limit = 20;		# length of one side of gradient, in iterations
  my $type = 'mandelbrot';	# default to mandelbrot
  my $power = 2;		# maximum power in equation
  my $num_threads = 4;
  
  my $file = shift or die "give me a file damnit!!!\n";
  
  open INPUT, "$file"; 
 
  while(<INPUT>) {
    
    chop;
    
    unless(/^;/) {
      s/[)(]//g;
      my @parm = split / /;
      if($parm[0] eq 'juliaseed') {
	@seed = @parm[1,2];
	$type = 'julia';
      } elsif($parm[0] eq 'maxiter') {
	$maxiter = $parm[1];
      } elsif($parm[0] eq 'view') {
	@view = @parm[1..4];
      } elsif($parm[0] eq 'formula' && $parm[1] =~ /mandel([0-9])/) {
	$power = $1;
      }
    }
    
  }
  
  $| = 1;
  
  #open OUT, ">output";
  
  my $ymin = $view[1]-$view[3]/2;
  my $ymax = $view[1]+$view[3]/2;
  my $xmin = $view[0]-$view[2]/2;
  my $xmax = $view[0]+$view[2]/2;
  my $ydelta = ($ymax - $ymin)/$pages_vert;
  my $xdelta = 10.5/8*$ydelta;
  
  #my $sliver = (($xmax-$xmin)/$xdelta - int(($xmax-$xmin)/$xdelta))*$xdelta/2.7;
  #$xmin -= $sliver;
  #$xmax += $sliver;
  
  my $func;
  
  if($type eq 'mandelbrot') {
    $func = "q^$power+c";
  } else {
    $func = "q^$power+[".$seed[0].' '.$seed[1].' 0 0]';
  }
  
  my $vi;
  
  for $vi (0..($pages_vert-1)) {
    my $this_ymin = $ymin+$vi*$ydelta;
    
    my $hi = 0;
    my $this_xmin;
    for($this_xmin = $xmin ; $this_xmin < $xmax ; $this_xmin = $xmin+$xdelta*$hi ) {
      my $this_xmax = $this_xmin+$xdelta >= $xmax ? $xmax : $this_xmin + $xdelta;
      my $this_ymax = $this_ymin + $ydelta;
      #    print "_____coords are $this_xmin $this_xmax $this_ymin $this_ymax\n";    
      my $ysize = $dpi*8;
      my $xsize = $this_xmin+$xdelta > $xmax ? ($this_xmax - $this_xmin)/($this_ymax - $this_ymin)*$ysize : $dpi*10.5;
      #    print "size is $xsize * $ysize\n";
      my $xspan = $this_xmax - $this_xmin;
      my $yspan = $this_ymax - $this_ymin;
      my $xcenter = ($this_xmax + $this_xmin)/2;
      my $ycenter = ($this_ymax + $this_ymin)/2;
      my $string = "~/src/bfe/bfe -ib 2 -nt $num_threads -pf pallete -p $power -l $limit -i $maxiter -b 500 -size $xsize $ysize -span $xspan $yspan -center $xcenter $ycenter 0 0 -hv 1 0 0 0 -vv 0 1 0 0 -f $hi.$vi.png -F '$func'";
      #  if ($x_do_only == $hi && $y_do_only == $vi) {
      print $string."\n";
      #    print "processing with bfe... ";
      #    system $string;
      #    print "done\n";
      #  }
      if($out) {
	#      print "post-processing with gimp... ";
	$string = "(gimp --no-data -n -b '(script-fu-print 1 \"\\\"$hi.$vi.png\\\"\" \"\\\"$hi.$vi.out\\\"\")' '(gimp-quit 0)' ; bzip2 $hi.$vi.out)";
	#      system $string;
	print "$string\n";
	#      print "done\n";
	#      system "bzip2 $hi$vi.out";
      }
      $hi++;
    }
  }
  
}
